<?php

/**
 * Settings Controller for Pressbackup.
 *
 * This Class provide a interface to display and manage Plugin settings
 * Also Manage the configurattion wizard
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @link			http://pressbackup.com
 * @package			controlers
 * @subpackage		controlers.settings
 * @since			0.1
 * @license			GPL v2 License
 */
 
class PressbackSettings {

	/**
	 * Redirect the user to de correct page
	 * If the user has not configured the plugin
	 * this function redirect directly to the config init page
	 *
	 * Note: this function is automaticaly called
	 * by pressing on settings menu link
	 */
	public function index()
	{
		global $pressbackup;

		$preferences = get_option('pressbackup_preferences');

		//misc
		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();
		$service = $misc->current_service();

		if ($preferences['pressbackup']['configured'] && $service && $service['id'] == 'Exp') {

			//$pressbackup->redirect(array('menu_type'=>'tools', 'controller'=>'principal', 'function'=>'dashboardExpress'));
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_credentials'));

		} else if ($preferences['pressbackup']['configured']) {

			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_credentials'));

		} else {

			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_init'));

		}
	}

	/**
	 * Start The configuration wizard
	 * Check all that all nedded libs are intalled
	 * Also check for perms on tmp folders
	 */
	function config_init() {

		global $pressbackup;
		$pressbackup->View->layout('principal');
		$preferences = get_option('pressbackup_preferences');

		//misc
		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$error = array();
		$disable_backup_files = false;
		$need_to_update_preferences = false;

		//check for
		if (strpos($_SERVER['SERVER_SOFTWARE'], 'iis') !== false) {
			$error['iss'] = __('Sorry your web Server (Windows IIS) is not complatible with Pressbackup. You are welcome to use Pressbackup, but we cannot provide any support or guarantees for your system','pressbackup');
		}

		//check for
		if (ini_get('safe_mode')) {
			$error['safe_mode'] = __('Safe mode is enabled. You would need to contact your hosting provider and ask them if it\'s  necessary for you to have safe_mode enabled. Most hosts should be able  to disable it for you, or give you a more specific answer','pressbackup');
			$disable_backup_files = true;
		}

		//check for Zip Creation
		$bin = $misc->checkShell('zip');
		if (!$bin) {
			$preferences['pressbackup']['compatibility']['zip'] = 10;
			$need_to_update_preferences = true;
		}
		if (!$bin && !class_exists('ZipArchive')) {
			$error['zip'] = __('You probably don\'t have the php-zip extension installed. You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		$blog_url = get_bloginfo( 'wpurl' );

		//Check for SSL
		$parsed_url = parse_url($blog_url);
		if($parsed_url['scheme'] == 'https'){
			$preferences['pressbackup']['compatibility']['background'] = 20;
			$need_to_update_preferences = true;
		}

		// Check for CURL
		if (!extension_loaded('curl')) {
			$error['curl'] = __('You probably don\'t have the php-curl extension installed.  You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		// Check CURL local
		$pressbackup->import('curl.php');
		$curl = new PressbackupCurl();

		$args = array(
			'url' => $blog_url
		);
		$curl->call($args);

		if($curl->response_code != 200) {
			$error['curl'] = __('You probably don\'t have permission to run curl locally.  You would need to contact your hosting provider and ask them if loopback HTTP requests are enabled.','pressbackup');
			$disable_backup_files = true;
		}

		//check for
		if (!class_exists('SimpleXMLElement')) {
			$error['xml'] = __('You probably don\'t have the SimpleXML extension installed.  You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		//tmp dir
		if (!file_exists($pressbackup->Path->Dir['PBKTMP']) && !mkdir($pressbackup->Path->Dir['PBKTMP'])) {
			$error['tmpdir'] = sprintf(__('Could not create the FileStore directory "%1$s". Please check the effective permissions.','pressbackup'), $pressbackup->Path->Dir['PBKTMP']);
			$disable_backup_files = true;
		} else {
			@chmod($pressbackup->Path->Dir['PBKTMP'], 0777);
		}

		//log dir
		if (!file_exists($pressbackup->Path->Dir['LOGTMP']) && !mkdir($pressbackup->Path->Dir['LOGTMP'])) {
			$error['logdir'] = sprintf(__('Could not create the FileStore directory "%1$s". Please check the effective permissions.','pressbackup'), $pressbackup->Path->Dir['LOGTMP']);
			$disable_backup_files = true;
		} else {
			@chmod($pressbackup->Path->Dir['LOGTMP'], 0777);
		}

		//validate use Cron
		$checkCron = get_option('pressbackup_check_cron');
		if ($checkCron == "fail") {
			$error['cron'] = __('Could not create a Cron Job Backup. Try it Again refreshing the page, or contact Pressbackup Support.', 'pressbackup');
			$disable_backup_files = true;
			update_option('pressbackup_check_cron', 'no checked');
		}
		elseif (!$disable_backup_files && $checkCron  != "success" ){
			$pressbackup->import('schedule_functions.php');
			$ps = new pressbackup_schedule();
			$ps->add_schedule('+4 seconds', 'pressbackup_check_cron_creation');

			$pressbackup->View->set('check_cron', 1);
		}

		//update preferences if its require
		if($need_to_update_preferences){
			update_option('pressbackup_preferences', $preferences);
		}

		//set errors
		if($pressbackup->Session->check('error') ){
			$error['auth'] = $pressbackup->Session->read('error');
			$pressbackup->Session->delete('error');
		}

		$pressbackup->View->set('error', $error);
		$pressbackup->View->set('disable_backup_files', $disable_backup_files);
	}

	/*
	 * 
	 */
	function config_decide(){
		global $pressbackup;

		if($_POST['data']['selected_product'] == 'free'){
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step1'));
		}elseif ($_POST['data']['selected_product'] == 'pay'){
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'configmembership_step1_save'));
		}
	}

	/**
	 * Show the form to select the service:
	 * S3, Pressbackup Pro, localhost, Dropbox, etc
	 */
	function config_step1() {

		global $pressbackup;
		$pressbackup->View->layout('principal');

		$service = false;
		if ($pressbackup->Session->check('error_msg')) {

			//save selected credentials && set error msg
			$service = $pressbackup->Session->read('service');
			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));

			$pressbackup->Session->delete('error');
			$pressbackup->Session->delete('service');
			$pressbackup->Session->delete('error_msg');
		}

		$pressbackup->View->set('service', $service);
		$pressbackup->View->set('settings', get_option('pressbackup_preferences'));
	}

	/**
	 * Store the credentials if they are correct
	 * or save the error and return to form page
	 */
	function config_step1_save() {

		global $pressbackup;

		//Exit if wrong parameters
		if (!isset($_POST['data']) || !$this->check_fields_step1($_POST['data'])) {
			$pressbackup->Session->write('error', true);
			$pressbackup->Session->write('service', $_POST['data']['service']);
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step1'));
		}

		$preferences = get_option('pressbackup_preferences');

		$preferences['pressbackup']['s3']['credential'] = '';
		$preferences['pressbackup']['pressbackuppro']['credential'] = '';
		$preferences['pressbackup']['pressexpress']['credential'] = '';
		$preferences['pressbackup']['dropbox']['credential'] = '';
		$preferences['pressbackup']['local']['path'] = '';


		if ($_POST['data']['service'] == 's3') {

			$PS3 = $_POST['data']['preferences']['S3'];

			$pressbackup->import('S3.php');
			$s3 = new PressbackupS3($PS3['accessKey'], $PS3['secretKey']);

			//check if bucket exists
			$buckets = @$s3->listBuckets();
			$the_bucket = $pressbackup->Config->read('S3.bucketname') . '-' . md5(uniqid(rand(), true));
			$create_bucket = true;

			for ($i = 0; $i < count($buckets); $i++) {
				if (strpos($buckets[$i], $pressbackup->Config->read('S3.bucketname')) !== false) {
					$the_bucket = $buckets[$i];
					$create_bucket = false;
					break;
				}
			}

			//create bucket if it not exist
			$region = ($PS3['region'] == 'EU') ? 'EU' : false;
			if ($create_bucket && !$s3->putBucket($the_bucket, PressbackupS3::ACL_PRIVATE, $region)) {
				$pressbackup->Session->write('error', true);
				$pressbackup->Session->write('error_msg', __("Unable to create a bucket on S3: Service temporarily unavailable. Please try it again later.",'pressbackup'));
				$pressbackup->Session->write('service', $_POST['data']['service']);
				$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step1'));
			}

			//values to save
			$preferences['pressbackup']['s3']['bucket_name'] = $the_bucket;
			$preferences['pressbackup']['s3']['region'] = $region;
			$preferences['pressbackup']['s3']['credential'] = base64_encode($PS3['accessKey'] . '|AllYouNeedIsLove|' . $PS3['secretKey']);

		} elseif ($_POST['data']['service'] == 'local') {

			$PLOCAL = $_POST['data']['preferences']['local'];
			$preferences['pressbackup']['local']['path'] = $PLOCAL['path'];

		}

		update_option('pressbackup_preferences', $preferences);


		if ($_POST['data']['service'] == 'dbox') {
			$pressbackup->import('dropbox.php');
			$dbox = new PressbackupDropbox();
			$dbox->auth(array('menu_type' => 'settings', 'controller' => 'settings', 'function' => 'dropbox_authorize'));
		} else {
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step2'));
		}
	}


	/**
	 * Show the form to enter prefered settings
	 * as backup type, schedule time, etc
	 */
	function config_step2() {

		global $pressbackup;
		$pressbackup->View->layout('principal');

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$preferences = get_option('pressbackup_preferences');

		//check if there are errors to show
		if ($pressbackup->Session->check('error')) {
			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));
			$pressbackup->Session->delete('error');
			$pressbackup->Session->delete('error_msg');
		}

		$pressbackup->View->set('settings', $preferences);
		$pressbackup->View->set('service', $misc->current_service());
	}

	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 */
	function config_step2_save() {

		global $pressbackup;
		$preferences = get_option('pressbackup_preferences');

		$pressbackup->import('schedule_functions.php');
		$ps = new pressbackup_schedule();

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		//exit if no data or invalid
		if (!isset($_POST['data']) || !$this->check_fields_step2($_POST['data']['preferences'])) {
			$pressbackup->Session->write('error', true);
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step2'));
		}

		//saving number of copies to store
		if (isset($_POST['data']['preferences']['copies'])) {
			$preferences['pressbackup']['backup']['copies'] = $_POST['data']['preferences']['copies'];
		} else {
			$preferences['pressbackup']['backup']['copies'] = 5;
		}

		//Check if davanced options were activated
		$advanced_activated = $_POST['data']['preferences']['backup_advanced']['activated'];

		// saving backup schedule time
		if ($advanced_activated) {
			$scheduled = $ps->min_time($_POST['data']['preferences']['backup_advanced']);
			$preferences['pressbackup']['backup']['time'] = $scheduled['time'];
		}
		elseif ( isset ($_POST['data']['preferences']['time']) ) {
			$preferences['pressbackup']['backup']['time'] = $_POST['data']['preferences']['time'];
		}
		else {
			$preferences['pressbackup']['backup']['time'] = 0;
		}

		//setting backup type
		if($advanced_activated){

			$curdate = strtotime($ps->midnight_UTC());
			$post = $_POST['data']['preferences']['backup_advanced'];

			$preferences['pressbackup']['backup_advanced']['activated'] = true;
			$preferences['pressbackup']['backup_advanced']['db']= array('time' => $post['db']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['plugins']= array('time' => $post['plugins']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['themes']= array('time' => $post['themes']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['uploads']= array('time' => $post['uploads']['time'], 'last_date' => $curdate);

			$preferences['pressbackup']['backup']['type'] = $ps->activated_types($preferences['pressbackup']['backup_advanced']);
		}else{
			$preferences['pressbackup']['backup_advanced']['activated'] = false;
			$preferences['pressbackup']['backup']['type'] = join(',', $_POST['data']['preferences']['type']);
		}

		$preferences['pressbackup']['configured'] = 1;

		update_option('pressbackup_preferences', $preferences);

		if ($misc->current_service()) {
			$ps->add_schedule('+1 seconds');sleep(1);
			$pressbackup->Session->write('dash.reload', 'dashboard');
		}

		$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
	}

	/**
	 * Store the credentials if they are correct
	 * or save the error and return to form page
	 */
	function configmembership_step1_save() {

		global $pressbackup;
		$preferences = get_option('pressbackup_preferences');

		$pressbackup->import('Pro.php');

		$pro = new pressbackup();

		$response = $pro->simpleauth();

		if(!$response) {
			$pressbackup->Session->write('error', __('Please register your site in https://pressbackup.com', 'pressback'));
			$pressbackup->redirect(array('controller'=>'settings', 'function'=>'config_init')); exit;
		}

		/**
		 * Config Default Presbackup Express
		 */

		$PPRO = array(
			'user' => $response['items'][0]['item']['username'],
			'key' => urldecode($response['items'][0]['item']['authkey']),
			'service' => $response['items'][1]['item']['package'],
		);

		if($PPRO['service'] == 'pro'){
			$preferences['pressbackup']['pressbackuppro']['credential'] = base64_encode($PPRO['user'] . '|AllYouNeedIsLove|' . $PPRO['key']);
		}else{
			$preferences['pressbackup']['pressexpress']['credential'] = base64_encode($PPRO['user'] . '|AllYouNeedIsLove|' . $PPRO['key']);
		}

		update_option('pressbackup_preferences', $preferences);

		$this->configmembership_step2_save();
	}

	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 */
	function configmembership_step2_save() {

		global $pressbackup;
		$preferences = get_option('pressbackup_preferences');

		$pressbackup->import('schedule_functions.php');
		$pb = new pressbackup_schedule();

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		/**
		 * Config Default Pressbackup Express
		 */
		$preferences['pressbackup']['backup']['copies'] = 7;
		$preferences['pressbackup']['backup']['time'] = 24;
		$preferences['pressbackup']['backup']['type'] = '7,5,3,1';

		$preferences['pressbackup']['configured'] = 1;

		update_option('pressbackup_preferences', $preferences);

		$pb->add_schedule('+1 seconds');sleep(1);

		$service = $misc->current_service();

		$pressbackup->import('Pro.php');
		$credentials=explode('|AllYouNeedIsLove|', base64_decode( $service['credentials'] ));
		$pro = new pressbackup($credentials[0], $credentials[1]);
		$pro->wasConfigured();

		if( $service['id'] == 'Pro' ){
			$pressbackup->Session->write('dash.reload', 'dashboard');
			$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
		}else{
			$pressbackup->Session->write('dash.reload', 'dashboardExpress');
			$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboardExpress'));
		}
	}


	/**
	 * Show the form to modify the service:
	 * S3, Pressbackup Pro, localhost, Dropbox, etc
	 */
	function config_credentials() {

		global $pressbackup;
		$pressbackup->View->layout('principal');

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$cs = $misc->current_service();
		$service = $cs['id'];

		if ($pressbackup->Session->check('error')) {

			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));
			$service = $pressbackup->Session->read('service');

			$pressbackup->Session->delete('error');
			$pressbackup->Session->delete('service');
			$pressbackup->Session->delete('error_msg');
		}

		$pressbackup->View->set('service', $service);
		$pressbackup->View->set('services', $misc->service());
		$pressbackup->View->set('settings', get_option('pressbackup_preferences'));

	}

	/**
	 * Store the modified credentials if they are correct
	 * or save the error and return to form page
	 */
	function config_credentials_save() {

		global $pressbackup;
		$pressbackup->import('backup_functions.php');
		$pressbackup->import('misc.php');
		$pressbackup->import('schedule_functions.php');

		if (!isset($_POST['data']) || !$this->check_fields_step1($_POST['data'])) {
			$pressbackup->Session->write('error', true);
			$pressbackup->Session->write('service', $_POST['data']['service']);
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_credentials'));
		}

		$preferences = get_option('pressbackup_preferences');

		$preferences['pressbackup']['s3']['credential'] = '';
		$preferences['pressbackup']['pressbackuppro']['credential'] = '';
		$preferences['pressbackup']['pressexpress']['credential'] = '';
		$preferences['pressbackup']['dropbox']['credential'] = '';
		$preferences['pressbackup']['local']['path'] = '';


		if ($_POST['data']['service'] == 's3') {

			$PS3 = $_POST['data']['preferences']['S3'];

			$pressbackup->import('S3.php');
			$s3 = new PressbackupS3($PS3['accessKey'], $PS3['secretKey']);

			//check if bucket exists
			$buckets = @$s3->listBuckets();
			$the_bucket = $pressbackup->Config->read('S3.bucketname') . '-' . md5(uniqid(rand(), true));
			$create_bucket = true;
			for ($i = 0; $i < count($buckets); $i++) {
				if (strpos($buckets[$i], $pressbackup->Config->read('S3.bucketname')) !== false) {
					$the_bucket = $buckets[$i];
					$create_bucket = false;
					break;
				}
			}

			//create bucket if it not exist
			$region = ($PS3['region'] == 'EU') ? 'EU' : false;
			if ($create_bucket && !$s3->putBucket($the_bucket, PressbackupS3::ACL_PRIVATE, $region)) {
				$pressbackup->Session->write('error', true);
				$pressbackup->Session->write('error_msg', __("Unable to create a bucket on S3: Service temporarily unavailable. Please try it again later.",'pressbackup'));
				$pressbackup->Session->write('service', $_POST['data']['service']);
				$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_credentials'));
			}

			//values to save
			$preferences['pressbackup']['s3']['bucket_name'] = $the_bucket;
			$preferences['pressbackup']['s3']['region'] = $region;
			$preferences['pressbackup']['s3']['credential'] = base64_encode($PS3['accessKey'] . '|AllYouNeedIsLove|' . $PS3['secretKey']);


		} elseif ($_POST['data']['service'] == 'pro') {

			$PPRO = $_POST['data']['preferences']['pressbackuppro'];
			$preferences['pressbackup']['pressbackuppro']['credential'] = base64_encode($PPRO['user'] . '|AllYouNeedIsLove|' . $PPRO['key']);

		} elseif ($_POST['data']['service'] == 'express') {

			$PEXP = $_POST['data']['preferences']['pressexpress'];
			$preferences['pressbackup']['pressexpress']['credential'] = base64_encode($PEXP['user'] . '|AllYouNeedIsLove|' . $PEXP['key']);
			$preferences['pressbackup']['backup_advanced']['activated'] = false;
			$preferences['pressbackup']['backup']['copies'] = 7;
			$preferences['pressbackup']['backup']['time'] = 24;
			$preferences['pressbackup']['backup']['type'] = '7,5,3,1';

		} elseif ($_POST['data']['service'] == 'local') {

			$PLOCAL = $_POST['data']['preferences']['local'];
			$preferences['pressbackup']['local']['path'] = $PLOCAL['path'];
		} 


		if ( $_POST['data']['service'] == 'no') {
			$preferences['pressbackup']['backup_advanced']['activated'] = false;
		}

		update_option('pressbackup_preferences', $preferences);

		$misc = new pressbackup_misc();
		$ps = new pressbackup_schedule();
		$service = $misc->current_service();

		//remove schedule if sleected none, o add it id selected other option
		if ($service) { $ps->add_schedule(); } else { $ps->remove_schedule(); }

		//inform about this action to pressbackup
		if ( in_array( $service['id'], array('Pro', 'Exp') )) {

			$pressbackup->import('Pro.php');

			$credentials=explode('|AllYouNeedIsLove|', base64_decode( $service['credentials'] ));
			$pro = new pressbackup($credentials[0], $credentials[1]);
			$pro->wasConfigured();
		}

		//redirect to dropbox if taht option was selected 
		if ($_POST['data']['service'] == 'dbox') {
			$pressbackup->import('dropbox.php');
			$dbox = new PressbackupDropbox();
			$dbox->auth(array('menu_type' => 'settings', 'controller' => 'settings', 'function' => 'dropbox_authorize'));
			exit;
		}

		$pressbackup->Session->write('general_msg', __('Service Saved!','pressbackup'));

		if ( $service['id'] == 'Exp'){
			$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboardExpress'));
		} else {
			$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
		}

	}

	/**
	 * Show the form to modify prefered settings
	 * as backup type, schedule time, etc
	 */
	function config_settings() {

		global $pressbackup;
		$pressbackup->View->layout('principal');
		$preferences = get_option('pressbackup_preferences');

		if ($pressbackup->Session->check('error')) {
			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));
			$pressbackup->Session->delete('error');
			$pressbackup->Session->delete('error_msg');
		}

		//Check for the current service and its credentials, if they are available
		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$pressbackup->View->set('settings', get_option('pressbackup_preferences'));
		$pressbackup->View->set('service', $misc->current_service());
	}

	/**
	 * Store the modified settings if they are correct
	 * or save the error and return to form page
	 */
	function config_settings_save() {

		global $pressbackup;
		$preferences = get_option('pressbackup_preferences');

		$pressbackup->import('schedule_functions.php');
		$ps = new pressbackup_schedule();

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		//exit if no data or invalid
		if (!isset($_POST['data']) || !$this->check_fields_step2($_POST['data']['preferences'])) {
			$pressbackup->Session->write('error', true);
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_settings'));
		}

		//saving number of copies to store
		if (isset($_POST['data']['preferences']['copies'])) {
			$preferences['pressbackup']['backup']['copies'] = $_POST['data']['preferences']['copies'];
		} else {
			$preferences['pressbackup']['backup']['copies'] = 5;
		}

		//Check if davanced options were activated
		$advanced_activated = $_POST['data']['preferences']['backup_advanced']['activated'];

		// saving backup schedule time
		if ($advanced_activated) {
			$scheduled = $ps->min_time($_POST['data']['preferences']['backup_advanced']);
			$preferences['pressbackup']['backup']['time'] = $scheduled['time'];
		}
		elseif ( isset ($_POST['data']['preferences']['time']) ) {
			$preferences['pressbackup']['backup']['time'] = $_POST['data']['preferences']['time'];
		}
		else {
			$preferences['pressbackup']['backup']['time'] = 0;
		}

		//setting backup type
		if($advanced_activated){
			$curdate = strtotime($ps->midnight_UTC());
			$post = $_POST['data']['preferences']['backup_advanced'];

			$preferences['pressbackup']['backup_advanced']['activated'] = true;
			$preferences['pressbackup']['backup_advanced']['db']= array('time' => $post['db']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['plugins']= array('time' => $post['plugins']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['themes']= array('time' => $post['themes']['time'], 'last_date' => $curdate);
			$preferences['pressbackup']['backup_advanced']['uploads']= array('time' => $post['uploads']['time'], 'last_date' => $curdate);

			$preferences['pressbackup']['backup']['type'] = $ps->activated_types($preferences['pressbackup']['backup_advanced']);
		}else{
			$preferences['pressbackup']['backup_advanced']['activated'] = false;
			$preferences['pressbackup']['backup']['type'] = join(',', $_POST['data']['preferences']['type']);
		}

		update_option('pressbackup_preferences', $preferences);

		if ($misc->current_service()) {
			$ps->add_schedule();
		}

		$pressbackup->Session->write('general_msg', __('Preferences Saved!','pressbackup'));
		$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));

	}


	/**
	 * Show the form to modify prefered conpatibility settings
	 */
	function config_compatibility() {

		global $pressbackup;
		$pressbackup->View->layout('principal');
		$preferences = get_option('pressbackup_preferences');

		$pressbackup->import('misc.php');
		$misc = new pressbackup_misc();

		$binaries = false;
		if ($misc->checkShell('php')) {
			$binaries = true;
		}

		$pressbackup->View->set('binaries', $binaries);
		$pressbackup->View->set('settings', $preferences);
	}

	/**
	 * Store the modified settings if they are correct
	 * or save the error and return to form page
	 */
	function config_compatibility_save() {

		global $pressbackup;

		$preferences = get_option('pressbackup_preferences');

		if (isset($_POST['data']['compatibility']['background'])) {
			$preferences['pressbackup']['compatibility']['background'] = $_POST['data']['compatibility']['background'];
		}

		if (isset($_POST['data']['compatibility']['zip'])) {
			$preferences['pressbackup']['compatibility']['zip'] = $_POST['data']['compatibility']['zip'];
		}

		update_option('pressbackup_preferences', $preferences);

		$pressbackup->Session->write('general_msg', __('Preferences Saved!','pressbackup'));
		$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
	}


	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 *
	 * @token_s string: token generated on revios step
	 */
	function dropbox_authorize($token_s) {
		global $pressbackup;

		$pressbackup->import('dropbox.php');
		$dbox = new PressbackupDropbox();

		$token = $_GET['oauth_token'];
		$tk = $dbox->get_access_token($token, $token_s);

		$preferences = get_option('pressbackup_preferences');

		$preferences['pressbackup']['dropbox']['credential'] = base64_encode($tk['oauth_token'] . '|AllYouNeedIsLove|' . $tk['oauth_token_secret']);
		update_option('pressbackup_preferences', $preferences);

		if($preferences['pressbackup']['configured']){
			$pressbackup->Session->write('general_msg', __('Service Saved!','pressbackup'));
			$pressbackup->redirect(array('menu_type' => 'tools', 'controller' => 'principal', 'function' => 'dashboard'));
		} else {
			$pressbackup->redirect(array('controller' => 'settings', 'function' => 'config_step2'));
		}
	}

	/**
	 * Check if credentials are valid
	 * for S3 or Pressbackup PRO account
	 *
	 * @args array: credentials sent via post
	 */
	function check_fields_step1($args) {

		global $pressbackup;

		if ($args['service'] == 's3') {
			$PS3 = $args['preferences']['S3'];

			//check empty data
			if (!isset($PS3['accessKey']) || empty($PS3['accessKey']) || !isset($PS3['secretKey']) || empty($PS3['secretKey'])) {
				$pressbackup->Session->write('error_msg', __('Empty Fields, please ty it again','pressbackup'));
				return false;
			}

			$pressbackup->import('S3.php');
			$s3 = new PressbackupS3($PS3['accessKey'], $PS3['secretKey']);

			//check if credentials are right
			if (($buckets = @$s3->listBuckets()) === false) {
				$pressbackup->Session->write('error_msg', __('Incorrect S3 access or secret Keys','pressbackup'));
				return false;
			}

			return true;
		} 

		if ($_POST['data']['service'] == 'pro') {

			$pressbackup->import('Pro.php');

			$PPRO = $args['preferences']['pressbackuppro'];

			$pro = new pressbackup();
			$response = $pro->simpleauth();

			if(!$response) {
				$pressbackup->Session->write('error_msg', __('Please register your site in https://pressbackup.com', 'pressback'));
				return false;
			}

			$res = array(
				'user' => $response['items'][0]['item']['username'],
				'key' => urldecode($response['items'][0]['item']['authkey']),
				'service' => $response['items'][1]['item']['package'],
			);

			if($res['service'] == 'pro'){
				$index = 'pressbackuppro';
			}else{
				$index = 'pressexpress';
			}

			$_POST['data']['service'] = $res['service'];
			$_POST['data']['preferences'][$index]['user'] = $res['user'];
			$_POST['data']['preferences'][$index]['key'] = $res['key'];

			return true;
		} 

		if ($args['service'] == 'local') {

			$PLOCAL = $args['preferences']['local'];

			$pressbackup->import('local.php');
			$local = new PressbackupLocal();

			if (!$local->check_exists($PLOCAL['path'])) {
				$pressbackup->Session->write('error_msg', __("The specified folder does not exist.",'pressbackup'));
				return false;
			}

			if (!$local->check_permissions($PLOCAL['path'])) {
				$pressbackup->Session->write('error_msg', __("No write permissions for the selected folder",'pressbackup'));
				return false;
			}

			return true;
		}

		return true;
	}

	/**
	 * Check if fields for Settings are valid.
	 * The files are type of backup, schedule time, etc
	 *
	 * @args array: all fields sent via post
	 */
	function check_fields_step2($args) {

		global $pressbackup;
		$pass = true;

		if(!isset($args['backup_advanced']['activated']) ){

			if (isset($args['time']) && !preg_match("/^[[:digit:]]+$/", $args['time'])) {
				$pressbackup->Session->write('error_msg', __('Specify the scheduled backup time range','pressbackup'));
				return false;
			}

			if (isset($args['copies']) && !preg_match("/^[[:digit:]]+$/", $args['copies'])) {
				$pressbackup->Session->write('error_msg', __('Specify the number of backups to be stored','pressbackup'));
				return false;
			}

			if (!isset($args['type']) || empty($args['type'])) {
				$pressbackup->Session->write('error_msg', __('Specify what you would like to backup','pressbackup'));
				return false;
			}

			for ($i = 0; $i < count($args['type']); $i++) {
				if (!preg_match("/^[[:digit:]]+$/", $args['type'][$i])) {
					$pressbackup->Session->write('error_msg', __('Empty/wrong Fields, please ty it again','pressbackup'));
					return false;
				}
			}

		}else {
			unset($args['backup_advanced']['activated']);
			$at_least_one = false;

			//check that all times received they are digit, and that at least one is activated,
			foreach($args['backup_advanced'] as $type=>$data){
				if (isset($data['time']) && !preg_match("/^[[:digit:]]+$/", $data['time'])) {
					$pressbackup->Session->write('error_msg', __('Specify the scheduled backup time range','pressbackup'));
					return false;
				}
				if($data['time'] != 0){
					$at_least_one = true;
				}
			}

			if(!$at_least_one){
				$pressbackup->Session->write('error_msg', __('Specify what you would like to backup','pressbackup'));
				return false;
			}
		}
		return true;
	}

	//cron checks

	function check_cron ()
	{
		update_option('pressbackup_check_cron', 'success');
	}
	
	function check_cron_fail ()
	{
		global $pressbackup;
		
		update_option('pressbackup_check_cron', 'fail');
		$pressbackup->redirect(array('menu_type'=>'settings', 'controller'=>'settings', 'function'=>'config_init'));
	}
	
	function check_cron_status ()
	{
		$respond = get_option('pressbackup_check_cron');
		if ($respond == 'success') {
			exit('{"status": "ok"}');
		} else {
			exit('{"status": "fail"}');
		}
	}
}
?>
