<?php

	class PressbackupLocal {

		private $local_folder = null;

		public function __construct($local_folder=null)
		{
			if ($local_folder) {
				$this->local_folder = rtrim( $local_folder, DS ) . DS;
			}
		}

		public function check_permissions($folder)
		{
			$folder = rtrim( $folder, DS ) . DS;
			return (@is_writable($folder) && @is_readable($folder));
		}

		public function check_exists($folder){
			return (@file_exists($folder));
		}

		public function is_empty($folder)
		{
			$folder = rtrim( $folder, DS );
			return (($files = @scandir($folder)) && count($files) <= 2); 
		}

		public function putFile($file)
		{
			return $this->putFile_fp($file);
		}

		public function putFile_fp($file)
		{
			//fix for no occidental characers, basename and pathinfo remove them from $file
			$pathinfo = explode(DS, $file);
			$name = array_pop($pathinfo);

			//Check if already exist the file on folder
			if(file_exists($this->local_folder . DS . $name)){
				return false;
			}

			return copy($file, $this->local_folder . DS . $name);
		}


		public function getFile($name,$saveon)
		{
			//Check if already exist the file on folder
			if(!@file_exists($this->local_folder . $name)){
				return false;
			}

			return @copy($this->local_folder . $name, $saveon);
		}

		public  function deleteFile($name)
		{
			//Check if already exist the file on folder
			if(!@file_exists($this->local_folder . $name)){
				return false;
			}

			return @unlink($this->local_folder . $name);
		}

		public function getFilesList(){
			$files = @scandir($this->local_folder);

			if(!$files){
				return false;
			}

			//Affects files that starts with special characters: -
			//unset($files[0]);//delete "." file
			//unset($files[1]);//delete ".." file

			$files_on_folder = array();

			foreach($files as $name){
				//check that the file is a valid backup generated by Pressbackup
				if (preg_match("/(.)+-backup_[7|5|3|1]+[(-5)|(-3)|(-1)]*_[a-zA-Z0-9]{13}.zip/i", $name) && !in_array($name,array('.','..'))){
					$file = $this->local_folder . $name;
					$files_on_folder[] = array("name" => $name,"time" => filemtime($file),"size" => filesize($file),"hash"=>"");
				}
			}
			return $files_on_folder;
		}
	}
?>
