<?php

 /**
 * Schedule Functions lib for Pressbackup.
 *
 * This Class provide the functionality to mannage the scheduled functions
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @link				http://pressbackup.com
 * @package		libs
 * @subpackage	libs.schedule
 * @since			0.1
 * @license			GPL v2 License
 */

	class pressbackup_schedule {

	//		Schedule functions
	//----------------------------------------------------------------------------------------
		function add_schedule($time=null, $task = 'pressback_backup_start_cronjob') {
			global $pressbackup;

			$pressbackup->import('misc.php');
			$misc = new pressbackup_misc();

			date_default_timezone_set($misc->timezone_string());

			//if time is passed as arg, then must be the new datetime to execute the next  schedule
			if($time){
				$start_time = strtotime($time);
			}
			else{//if $time  it's not seted
				$start_time = $this->calculate_the_next_execution();
			}
			$this->remove_schedule($task);

			@wp_schedule_single_event($start_time, $task);
			return true;
		}

		function calculate_the_next_execution()
		{
			$preferences = get_option('pressbackup_preferences');

			if($preferences['pressbackup']['backup_advanced']['activated']){
				//obtain the time of more frequent job, and adding it to the last time it was executed
				$scheduled = $this->min_time();
				$last_date = $scheduled['last_date'];
				$time_step = $scheduled['time'] * (60 * 60);
			}else{
				$last_date = strtotime($this->midnight_UTC());
				$time_step = $preferences['pressbackup']['backup']['time'] * (60 * 60);
			}

			$curdate = strtotime('today 23:59:59');
			$start_time = $last_date + $time_step;

			//repeat above step, until start_time be greater than curdate
			while($start_time < $curdate){
				$start_time += $time_step;
			}

			return $start_time;
		}

		function remove_schedule($task = 'pressback_backup_start_cronjob')
		{
			@wp_clear_scheduled_hook($task);
			return true;
		}

		function find_something_to_do($scheduler)
		{
			if(!$scheduler['activated']){return false;}

			$jobs_to_do = '';

			if($this->is_scheduled($scheduler['db']['time'],$scheduler['db']['last_date'])){
				if($jobs_to_do != ''){$jobs_to_do .= ',';}
				$jobs_to_do .= '7';
			}
			if($this->is_scheduled($scheduler['themes']['time'],$scheduler['themes']['last_date'])){
				if($jobs_to_do != ''){$jobs_to_do .= ',';}
				$jobs_to_do .= '5';
			}
			if($this->is_scheduled($scheduler['plugins']['time'],$scheduler['plugins']['last_date'])){
				if($jobs_to_do != ''){$jobs_to_do .= ',';}
				$jobs_to_do .= '3';
			}
			if($this->is_scheduled($scheduler['uploads']['time'],$scheduler['uploads']['last_date'])){
				if($jobs_to_do != ''){$jobs_to_do .= ',';}
				$jobs_to_do .= '1';
			}

			return $jobs_to_do;
		}

		function is_scheduled($rate,$last_date){
			if($rate == '0'){ return false;}

			$curdate = strtotime('now');

			//calculates the time elapsed since the last execution so far
			$curdate_aux = intval($curdate);
			$last_date = intval($last_date);
			$elapsed_time = intval(($curdate_aux - $last_date)/3600);

			//must be executed
			if($elapsed_time >= $rate){
				return true;
			}

			//must not be executed
			return false;
		}

		function was_scheduled_for($rate,$last_date){
			//pass the rate to seconds
			$rate = $rate * 3600;

			//calculate the execution date scheduled
			$scheduled_date = $last_date + $rate;

			return $scheduled_date;
		}

		function update_last_date($jobs_to_do = null){

			if(!$jobs_to_do){ return false; }

			$preferences= get_option('pressbackup_preferences');


			$jobs_to_do = explode(',',$jobs_to_do);

			$curdate = strtotime('now');

			$as = $preferences['pressbackup']['backup_advanced'];

			if(in_array('7',$jobs_to_do)){
				//obtain the last scheduled time for DB.
				$last_date =  $this->was_scheduled_for($as['db']['time'],$as['db']['last_date']);
				//calculate the new $last_date. 
				while($last_date < $curdate - ($as['db']['time'] * (60 * 60))){
					$last_date = $last_date + ($as['db']['time'] * (60 * 60));
				}
				$preferences['pressbackup']['backup_advanced']['db']['last_date'] = $last_date;
			}
			if(in_array('5',$jobs_to_do)){
				//obtain the last scheduled time for THEMES.
				$last_date = $this->was_scheduled_for($as['themes']['time'],$as['themes']['last_date']);
				//calculate the new $last_date.
				while($last_date < $curdate - ($as['themes']['time'] * (60 * 60))){
					$last_date = $last_date + ($as['themes']['time'] * (60 * 60));
				}

				$preferences['pressbackup']['backup_advanced']['themes']['last_date'] = $last_date;
			}
			if(in_array('3',$jobs_to_do)){
				//obtain the last scheduled time for PLUGINS.
				$last_date = $this->was_scheduled_for($as['plugins']['time'],$as['plugins']['last_date']);
				//calculate the new $last_date.
				while($last_date < $curdate - ($as['plugins']['time'] * (60 * 60))){
					$last_date = $last_date + ($as['plugins']['time'] * (60 * 60));
				}
				$preferences['pressbackup']['backup_advanced']['plugins']['last_date'] = $last_date;
			}
			if(in_array('1',$jobs_to_do)){
				//obtain the last scheduled time for UPLOADS.
				$last_date = $this->was_scheduled_for($as['uploads']['time'],$as['uploads']['last_date']);
				//calculate the new $last_date.
				while($last_date < $curdate - ($as['uploads']['time'] * (60 * 60))){
					$last_date = $last_date + ($as['uploads']['time'] * (60 * 60));
				}
				$preferences['pressbackup']['backup_advanced']['uploads']['last_date'] = $last_date;
			}
			update_option('pressbackup_preferences', $preferences); 
		}

		
		/**
		* return the next time of the next backup
		*/
		function min_time($types = null){
			if(!$types){
				$preferences= get_option('pressbackup_preferences');
				$types = $preferences['pressbackup']['backup_advanced'];
			}
			if(isset($types['activated'])){
				unset($types['activated']);
			}

			$min['time'] = 99999999;
			foreach($types as $type=>$settings){
				if($settings['time'] && $settings['time'] < $min['time'] ){ $min = $settings; }//return the time and last_date
			}

			//if(!isset($min['last_date'])){$min['last_date'] = strtotime('now');}
			if(!isset($min['last_date'])){$min['last_date'] = strtotime($this->midnight_UTC());}

			return ($min['time'] == 99999999)?false:$min;
		}

		public function midnight_UTC (){
			$tz = date_default_timezone_get();
			date_default_timezone_set('UTC');
			$midnight_tz = strtotime('today 00:00:00');
			date_default_timezone_set($tz);
			$midnight_UTC_tz = date('d M Y H:i:s', $midnight_tz);

			return $midnight_UTC_tz;
		} 

		/**
		* -  params  -
		* @types: array with advanced backup preferences
		* @return: an string with the next types of backups that must be done: example "1,3,5".
		*/
		function next_scheduled_jobs($types=null){

			if(!$types){
				$preferences= get_option('pressbackup_preferences');
				$types = $preferences['pressbackup']['backup_advanced'];
			}
			if(isset($types['activated'])){
				unset($types['activated']);
			}

			$next_dates;
			$next_date = '';
			foreach($types as $key=>$type){
				if($type['time'] == 0){ continue; }

				$next_dates[$key] = $this->was_scheduled_for($type['time'],$type['last_date']);
				if($next_execution == ''){
					$next_execution = $next_dates[$key];
				}else if($next_execution > $next_dates[$key]){
					$next_execution = $next_dates[$key];
				}
			}

			$result_type = array();
			foreach($next_dates as $key=>$date){
				if($next_execution == $date){
					$result_type[] = $key;
				}
			}

			return  str_replace( array('uploads', 'plugins', 'themes', 'db'), array('1', '3', '5', '7') , join( ',', $result_type) );
		}

		/**
		* -  params  -
		* @types: array with advanced backup preferences
		* @return: an string with the types of backups enables: example "1,3,5".
		*/
		function activated_types($types=null){

			if(!$types){
				$preferences= get_option('pressbackup_preferences');
				$types = $preferences['pressbackup']['backup_advanced'];
			}

			if(isset($types['activated'])){ unset($types['activated']); }

			$activated = array();
			foreach($types as $type=>$settings){
				if($settings['time']){ $activated[] = $type; }
			}

			return  str_replace( array('uploads', 'plugins', 'themes', 'db'), array('1', '3', '5', '7') , join(',', $activated) );
		}

}
?>
